@@ -103,4 +103,24 @@ def find_file(dir, suffix=".npy"):
             file_name, suffix_name = os.path.splitext(file)
             if suffix_name == suffix:
                 file_list.append(file)
-    return file_list
\ No newline at end of file
+    return file_list
+
+def make_directory(path: str):
+    """Make directory."""
+    if path is None or not isinstance(path, str) or path.strip() == "":
+        raise TypeError("Input path '{}' is invalid type".format(path))
+
+    path = os.path.realpath(path)
+
+    if os.path.exists(path):
+        real_path = path
+    else:
+        try:
+            permissions = os.R_OK | os.W_OK | os.X_OK
+            os.umask(permissions << 3 | permissions)
+            mode = permissions << 6
+            os.makedirs(path, mode=mode, exist_ok=True)
+            real_path = path
+        except PermissionError:
+            raise TypeError("No write permission on the directory `{path}`.")
+    return real_path
\ No newline at end of file
