@@ -14,11 +14,14 @@
 # ============================================================================
 
 
-weight_map = {"torch.nn.BatchNorm1d":{'weight':'gamma', 'bias':'beta', 'running_mean':'moving_mean' ,'running_var':'moving_variance'},
+weight_name_map = {"torch.nn.BatchNorm1d":{'weight': 'gamma', 'bias': 'beta', 'running_mean': 'moving_mean' , 'running_var': 'moving_variance'},
     "torch.nn.BatchNorm2d":{'weight':'gamma', 'bias':'beta', 'running_mean':'moving_mean' ,'running_var':'moving_variance'},
     "torch.nn.BatchNorm3d":{'weight':'gamma', 'bias':'beta', 'running_mean':'moving_mean' ,'running_var':'moving_variance'},
     "torch.nn.Embedding":{'weight':'embedding_table'},
     "torch.nn.GroupNorm":{'weight':'gamma', 'bias': 'beta'},
     "torch.nn.LayerNorm":{'weight': 'gamma', 'bias': 'beta'},
     "torch.nn.PReLU":{'weight':'a'},
-}
+                   }
+
+weight_value_map = {"torch.nn.Conv1d":{'weight': 'troubleshooter.migrator.mapping_relation.weight_adapter.trans_conv1d_weight_value'}
+                    }
\ No newline at end of file
